@@ -23,6 +23,8 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 
+#include <sched.h>
+
 #include <array>
 #include <atomic>
 #include <cstddef>
@@ -42,6 +44,7 @@
 
 #include "nd-except.hpp"
 #include "nd-sha1.h"
+#include "nd-types.hpp"
 
 constexpr unsigned ND_SHA1_BUFFER = 4096;
 typedef std::array<uint8_t, SHA1_DIGEST_LENGTH> ndDigest;
@@ -90,10 +93,6 @@ constexpr size_t _ND_LOG_FILE_STAMP_SIZE = sizeof(
 #define ndEnumCast(T, n) \
     static_cast<typename std::underlying_type<T>::type>(T::n)
 
-void *nd_mem_alloc(size_t size);
-
-void nd_mem_free(void *ptr);
-
 class ndLogBuffer : public std::streambuf
 {
 public:
@@ -188,25 +187,24 @@ public:
 
     friend std::ostream &
     operator<<(std::ostream &os, const ndLogFormat &f) {
-        using namespace std;
-        ios old_state(nullptr);
+        std::ios old_state(nullptr);
         old_state.copyfmt(os);
-        os << setw(f.width) << setprecision(f.precision);
+        os << std::setw(f.width) << std::setprecision(f.precision);
 
         switch (f.format) {
         case Format::BYTES:
             if (f.value >= 1099511627776.0f) {
                 os << (f.value / 1099511627776.0f)
-                   << setw(0) << " TiB";
+                   << std::setw(0) << " TiB";
             }
             else if (f.value >= 1073741824.0f) {
-                os << (f.value / 1073741824.0f) << setw(0) << " GiB";
+                os << (f.value / 1073741824.0f) << std::setw(0) << " GiB";
             }
             else if (f.value >= 1048576.0f) {
-                os << (f.value / 1048576.0f) << setw(0) << " MiB";
+                os << (f.value / 1048576.0f) << std::setw(0) << " MiB";
             }
             else if (f.value >= 1024.0f) {
-                os << (f.value / 1024.0f) << setw(0) << " KiB";
+                os << (f.value / 1024.0f) << std::setw(0) << " KiB";
             }
             else {
                 os << f.value;
@@ -216,16 +214,16 @@ public:
         case Format::PACKETS:
             if (f.value >= 1000000000000.0f) {
                 os << (f.value / 1000000000000.0f)
-                   << setw(0) << " TP";
+                   << std::setw(0) << " TP";
             }
             else if (f.value >= 1000000000.0f) {
-                os << (f.value / 1000000000.0f) << setw(0) << " GP";
+                os << (f.value / 1000000000.0f) << std::setw(0) << " GP";
             }
             else if (f.value >= 1000000.0f) {
-                os << (f.value / 1000000.0f) << setw(0) << " MP";
+                os << (f.value / 1000000.0f) << std::setw(0) << " MP";
             }
             else if (f.value >= 1000.0f) {
-                os << (f.value / 1000.0f) << setw(0) << " KP";
+                os << (f.value / 1000.0f) << std::setw(0) << " KP";
             }
             else {
                 os << f.value;
@@ -259,12 +257,6 @@ void nd_dprintf(const char *format, ...);
 void nd_dprintf(const char *format, va_list ap);
 void nd_flow_printf(const char *format, ...);
 
-#ifdef _ND_ENABLE_NDPI_DEBUG
-void nd_ndpi_debug_printf(uint32_t protocol, void *ndpi,
-  ndpi_log_level_t level, const char *file,
-  const char *func, unsigned line, const char *format, ...);
-#endif
-
 void nd_ltrim(std::string &s, unsigned char c = 0);
 void nd_rtrim(std::string &s, unsigned char c = 0);
 void nd_trim(std::string &s, unsigned char c = 0);
@@ -298,10 +290,10 @@ bool nd_is_ipaddr(const char *ip);
 
 void nd_private_ipaddr(uint8_t index, struct sockaddr_storage &addr);
 
-bool nd_load_uuid(std::string &uuid,
-  const std::string &path, size_t length);
-bool nd_save_uuid(const std::string &uuid,
-  const std::string &path, size_t length);
+bool nd_load_uuid(
+  std::string &uuid, const std::string &path, size_t length);
+bool nd_save_uuid(
+  const std::string &uuid, const std::string &path, size_t length);
 
 void nd_seed_rng(void);
 
@@ -326,6 +318,7 @@ int nd_ifreq(const std::string &name, unsigned long request,
   struct ifreq *ifr);
 
 void nd_basename(const std::string &path, std::string &base);
+void nd_pathname(const std::string &path, std::string &base);
 
 pid_t nd_is_running(pid_t pid, const std::string &exe_base);
 pid_t nd_load_pid(const std::string &pidfile);
@@ -402,6 +395,17 @@ protected:
     timer_t id;
 };
 
+class ndCPUSet
+{
+public:
+    ndCPUSet(bool empty = false);
+    ndCPUSet(const std::string &cpu_list);
+
+    void Get(cpu_set_t &cpu_set);
+
+    std::vector<int> cpu_set;
+};
+
 void nd_get_ip_protocol_name(int protocol, std::string &result);
 
 int nd_glob(const std::string &pattern,
@@ -600,3 +604,132 @@ struct ndEnumHasher {
         return ss_hash;
     }
 };
+
+struct ndArrayHasher {
+    template <class T, size_t N>
+    std::size_t operator()(const std::array<T, N>& a) const {
+        std::hash<T> hasher;
+        std::size_t ss_hash = 0;
+        for (const auto& e : a) {
+            ss_hash ^= hasher(e) + 0x9e3779b9 + (ss_hash << 6) + (ss_hash >> 2);
+        }
+
+        return ss_hash;
+    }
+};
+
+std::ostream &operator<<(std::ostream &stream, ndApp::Id id);
+
+#if defined(__linux__)
+int nd_set_netns(const std::string &ns);
+#endif
+
+inline void nd_sprintf(std::string &dst, const char *format) {
+    dst.assign(format);
+}
+
+template<typename... Args>
+void nd_sprintf(std::string &dst, const char *format, Args... args) {
+    int length = std::snprintf(nullptr, 0, format, args...);
+    if (length == 0)
+        throw ndException("format string is zero length");
+
+    char *buffer = new char[length + 1];
+    std::snprintf(buffer, length + 1, format, args...);
+
+    dst.assign(buffer);
+    delete[] buffer;
+}
+
+#if defined(__linux__)
+void nd_get_memusage(const std::string &process, size_t &vm_kb, size_t &rss_kb);
+#endif
+
+uid_t nd_get_user_id(const std::string &name);
+gid_t nd_get_group_id(const std::string &name);
+
+void nd_drop_privileges(const std::string &user, const std::string &group);
+
+void nd_enable_coredumps(bool enable = true);
+
+#define ndTod_to_days(days) static_cast<ndTimeOfDay::Days>(days)
+#define ndTod_to_policy(policy) static_cast<uint8_t>(policy)
+
+class ndTimeOfDay {
+public:
+    typedef uint8_t Days;
+
+    enum class Day : Days {
+        NONE = 0,
+        SUN = (1 << 0),
+        MON = (1 << 1),
+        TUE = (1 << 2),
+        WED = (1 << 3),
+        THU = (1 << 4),
+        FRI = (1 << 5),
+        SAT = (1 << 6),
+        ALL = (SUN | MON | TUE | WED | THU | FRI | SAT),
+    };
+
+    enum class Policy : uint8_t {
+        INCLUDE,
+        EXCLUDE
+    };
+
+    ndTimeOfDay(time_t tv_update_freq, const std::string &timezone = "") {
+        SetTimezone(tv_update_freq, timezone);
+    }
+
+    ndTimeOfDay(const nlohmann::json &jconfig) {
+        Load(*this, jconfig);
+    }
+    ndTimeOfDay(
+        const std::string &time_start = "00:00",
+        const std::string &time_end = "23:59",
+        Days days = ndTod_to_days(Day::ALL),
+        Policy policy = Policy::INCLUDE) {
+        Load(*this, time_start, time_end, days, policy);
+    }
+
+    static void Load(ndTimeOfDay &tod,
+        const nlohmann::json &jconfig);
+    static void Load(ndTimeOfDay &tod,
+        const std::string &time_start = "00:00",
+        const std::string &time_end = "23:59",
+        Days days = ndTod_to_days(Day::ALL),
+        Policy policy = Policy::INCLUDE);
+
+    static time_t ParseTime(const std::string &time_string);
+
+    static void Describe(const ndTimeOfDay &tod, std::string &desc);
+
+    void SetTimezone(time_t tv_update_freq, const std::string &timezone) {
+        tv_last_update = 0;
+        this->tv_update_freq = tv_update_freq;
+        this->timezone = timezone;
+
+        TickUpdate();
+    }
+
+    void UpdateTimezone(void);
+
+    time_t TickUpdate(void);
+
+    bool operator==(const time_t tv) const;
+    bool operator!=(const time_t tv) const {
+        return (! ((*this) == tv));
+    }
+
+    bool valid = { false };
+    time_t time_start = { 0 };
+    time_t time_end = { 3600 * 23 + 59 };
+    Days days = { ndTod_to_days(Day::ALL) };
+    Policy policy = { Policy::INCLUDE };
+    time_t tv_update_freq = { 0 };
+    time_t tv_last_update = { 0 };
+    time_t tv_tick = { 0 };
+    std::string timezone;
+    std::string timezone_abbrv;
+    bool is_dst = { false };
+    int utc_offset = { 0 };
+};
