@@ -1969,44 +1969,63 @@ void ndTimeOfDay::Load(ndTimeOfDay &tod,
 
 time_t ndTimeOfDay::ParseTime(const string &time_string) {
     size_t p1 = time_string.find_first_of(":");
-    if (p1 == string::npos)
-        throw ndException("%s: invalid time_string", __PRETTY_FUNCTION__);
+    if (p1 == string::npos) {
+        throw ndException(
+            "%s: invalid time_string, must be 00:00[:00]",
+            __PRETTY_FUNCTION__);
+    }
 
     time_t time_hour = (time_t)strtold(
         time_string.substr(0, p1).c_str(), nullptr);
 
-    if (time_hour > 23)
-        throw ndException("%s: invalid hour (> 23)", __PRETTY_FUNCTION__);
+    if (time_hour > 23) {
+        throw ndException(
+            "%s: invalid hour (%lu > 23)",
+            __PRETTY_FUNCTION__, time_hour);
+    }
 
-    size_t p2 = time_string.find(":", p1 + 1);
+    size_t p2 = time_string.find(":", ++p1);
 
-    time_t time_min = 0;
+    time_t time_min;
 
     if (p2 == string::npos) {
         time_min = (time_t)strtold(
-            time_string.substr(p1 + 1).c_str(), nullptr);
+            time_string.substr(p1).c_str(), nullptr);
     }
     else {
-        if (p2 - p1 != 2)
-            throw ndException("%s: minutes must be 2 digits", __PRETTY_FUNCTION__);
+        if (p2 - p1 != 2) {
+            throw ndException(
+                "%s: minutes must be 2 digits (%lu digits found)",
+                __PRETTY_FUNCTION__, p2 - p1);
+        }
 
         time_min = (time_t)strtold(
-            time_string.substr(p1 + 1, 2).c_str(), nullptr);
+            time_string.substr(p1, 2).c_str(), nullptr);
     }
 
-    if (time_min > 59)
-        throw ndException("%s: invalid minute (> 59)", __PRETTY_FUNCTION__);
+    if (time_min > 59) {
+        throw ndException(
+            "%s: invalid minute (%lu > 59)",
+            __PRETTY_FUNCTION__, time_min);
+    }
 
     time_t time_sec = 0;
 
     if (p2 != string::npos) {
         time_sec = (time_t)strtold(
-            time_string.substr(p2 + 1).c_str(), nullptr);
+            time_string.substr(++p2).c_str(), nullptr);
 
-        if (time_sec > 59)
-            throw ndException("%s: invalid seconds (> 59)", __PRETTY_FUNCTION__);
+        if (time_sec > 59) {
+            throw ndException(
+                "%s: invalid seconds (%lu > 59)",
+                __PRETTY_FUNCTION__, time_sec);
+        }
     }
-
+#if 0
+    nd_dprintf("%s: %s == %lu seconds.\n",
+        __PRETTY_FUNCTION__, time_string.c_str(),
+        (time_hour * 3600) + (time_min * 60) + time_sec);
+#endif
     return (time_hour * 3600) + (time_min * 60) + time_sec;
 }
 
